/*
Description:
A widget that dynamically creates a tile view using List Tiles for a list of incoming data.
The widget accepts a title, subtitle, leading and trailing icons for each list tile along.

Imports:
import 'package:flutter_widgets_library/utils/widget-classesUsed.dart';
import 'custom-listTile.dart';

Usage:
- The widget returns the index value and the title of a tapped tile using the callback function.

- Defining the widget objects and parameters 
    List<CustomTileData> listData = [
      CustomTileData(
          leading: Icon(Icons.search),
          title: 'Search',
          subtitle: 'Search Bar',
          trailingTopText: 'Search here',
          trailingBottomText: 'Search Widget',
          enabled: true,
          selected: true),
      CustomTileData(
          leading: Icon(Icons.home),
          title: 'Home',
          subtitle: 'Home Button',
          trailingTopText: 'Home',
          trailingBottomText: 'Home Widget',
          enabled: true,
          selected: false),
      CustomTileData(
          leading: Icon(Icons.alarm),
          title: 'Clock',
          subtitle: 'Set Alarms',
          trailingTopText: 'Alarms',
          trailingBottomText: 'Clock Widget',
          enabled: true,
          selected: false)
    ];
    CustomListTileParameters tileParameters = new CustomListTileParameters(
        dense: true,
        isThreeLine: true,
        fontWeight: FontWeight.bold,
        fontSize: 14);

    onListTapped(int index, String title) {
      print(index);
      print(title);
    }

- Calling the widget object
    child: CustomListTile(listData, tileParameters, onListTapped)
    
Author:
Rishabh Thakur
*/

import 'package:flutter/material.dart';
import 'consts.dart' as consts;
import 'widget-classesUsed.dart';
import 'package:quiver/iterables.dart';
import 'theme.dart';

Widget CustomListTile(List<CustomTileData> data, CustomListTileParameters param,
    Function(int, String) callback) {
  if (data == null || data.length == 0 || param == null) {
    return Column(
      children: <Widget>[
        Container(
          height: consts.heightEighty,
          decoration: BoxDecoration(
            color: MediumGreyThemeData().canvasColor,
            borderRadius: BorderRadius.circular(consts.circular_radius),
          ),
          child: ListTile(
            title: Text(consts.listTitle),
            subtitle: Text(consts.listSubtitle),
            leading: Card(child: Icon(Icons.add_box)),
            isThreeLine: true,
          ),
        ),
        Container(
          height: consts.heightEighty,
          decoration: BoxDecoration(
            color: MediumGreyThemeData().canvasColor,
            borderRadius: BorderRadius.circular(consts.circular_radius),
          ),
          child: ListTile(
            title: Text(consts.listTitle),
            subtitle: Text(consts.listSubtitle),
            leading: Card(child: Icon(Icons.add_box)),
            isThreeLine: true,
          ),
        ),
      ],
    );
  } else {
    return SingleChildScrollView(
        child: Column(
            children: enumerate(data)
                .map((key) => Padding(
                      padding: const EdgeInsets.all(consts.heightFive),
                      child: Container(
                        height: (param.listHeight != null)
                            ? param.listHeight
                            : consts.heightEighty,
                        decoration: BoxDecoration(
                          color: (param.listBgColor != null)
                              ? param.listBgColor
                              : MediumGreyThemeData().canvasColor,
                          borderRadius: BorderRadius.circular(
                              param.circularRadius != null
                                  ? param.circularRadius
                                  : consts.circular_radius),
                        ),
                        child: (ListTile(
                          leading: Container(
                              height: (param.iconHeight != null)
                                  ? param.iconHeight
                                  : consts.heightFifty,
                              width: (param.iconWidth != null)
                                  ? param.iconHeight
                                  : consts.heightFifty,
                              color: (param.iconBgColor != null)
                                  ? param.iconBgColor
                                  : MediumGreyThemeData().cardColor,
                              child: key.value.leading),
                          title: Text(
                            key.value.title,
                            style: TextStyle(
                                fontSize: param.fontSize,
                                color: param.textColor,
                                fontWeight: param.fontWeight,
                                fontFamily: param.fontFamily),
                            overflow: TextOverflow.ellipsis,
                          ),
                          subtitle: Text(key.value.subtitle,
                              style: TextStyle(
                                  fontSize: param.fontSize,
                                  color: param.textColor,
                                  fontWeight: param.fontWeight,
                                  fontFamily: param.fontFamily),
                              overflow: TextOverflow.ellipsis),
                          trailing: Column(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: <Widget>[
                              Text(
                                  key.value.trailingTopText != null
                                      ? key.value.trailingTopText
                                      : '',
                                  style: TextStyle(
                                      fontSize: param.fontSize,
                                      color: param.textColor,
                                      fontWeight: param.fontWeight,
                                      fontFamily: param.fontFamily),
                                  overflow: TextOverflow.ellipsis),
                              Text(
                                  key.value.trailingBottomText != null
                                      ? key.value.trailingBottomText
                                      : '',
                                  style: TextStyle(
                                      fontSize: param.fontSize,
                                      color: param.textColor,
                                      fontWeight: param.fontWeight,
                                      fontFamily: param.fontFamily),
                                  overflow: TextOverflow.ellipsis),
                            ],
                          ),
                          isThreeLine: true,
                          onTap: () {
                            return callback(key.index, key.value.title);
                          },
                          onLongPress: () {
                            return callback(key.index, key.value.title);
                          },
                          enabled: (key.value.enabled != null)
                              ? key.value.enabled
                              : true,
                          selected: (key.value.selected != null)
                              ? key.value.selected
                              : false,
                        )),
                      ),
                    ))
                .toList()));
  }
}
